package trade_analysis

import (
	"eta/eta_index_lib/models"
	tradeAnalysisModel "eta/eta_index_lib/models/trade_analysis"
	"eta/eta_index_lib/utils"
	"fmt"
	"math"
	"time"
)

// CheckAnalysisCorrelationExtraConfig 校验相关性表格配置
func CheckAnalysisCorrelationExtraConfig(extraConfig tradeAnalysisModel.CorrelationTableExtraConfig) (pass bool, tips string) {
	if extraConfig.BaseEdbInfoId <= 0 {
		tips = "请选择标的指标"
		return
	}
	if extraConfig.Exchange == "" {
		tips = "请选择交易所"
		return
	}
	if extraConfig.ClassifyName == "" {
		tips = "请选择品种"
		return
	}
	if len(extraConfig.CompanyNames) == 0 {
		tips = "请选择期货公司"
		return
	}
	if len(extraConfig.CompanyNames) > 5 {
		tips = "最多选择5个期货公司"
		return
	}
	typeArr := []int{tradeAnalysisModel.ContractQueryTypeTop, tradeAnalysisModel.ContractQueryTypeTop2, tradeAnalysisModel.ContractQueryTypeTop3, tradeAnalysisModel.ContractQueryTypeAll, tradeAnalysisModel.ContractQueryTypeTotal}
	if !utils.InArrayByInt(typeArr, extraConfig.ContractType) {
		tips = "请选择正确的合约"
		return
	}
	positionArr := []int{tradeAnalysisModel.ContractPositionBuy, tradeAnalysisModel.ContractPositionSold, tradeAnalysisModel.ContractPositionPureBuy}
	if !utils.InArrayByInt(positionArr, extraConfig.ContractPosition) {
		tips = "请选择正确的合约方向"
		return
	}
	if len(extraConfig.RollConfig) == 0 {
		tips = "请设置滚动相关性配置"
		return
	}
	for _, v := range extraConfig.RollConfig {
		if v.CalculateValue < 2 || v.CalculateValue > 200 {
			tips = "计算窗口请输入2-200的整数"
			return
		}
		if v.LeadValue < 0 || v.LeadValue > 60 {
			tips = "B领先A请输入0-60的整数"
			return
		}
	}
	if extraConfig.PredictRatio < 0 || extraConfig.PredictRatio > 1 {
		tips = "请输入正确的估计参数"
		return
	}
	pass = true
	return
}

// GetCorrelationTableRowsDataByConfig 根据配置获取相关性表格数据
func GetCorrelationTableRowsDataByConfig(tableConfig tradeAnalysisModel.CorrelationTableExtraConfig) (tables []*tradeAnalysisModel.CorrelationTableData, err error) {
	tables = make([]*tradeAnalysisModel.CorrelationTableData, 0)

	// 获取标的指标数据
	baseDateData := make(map[time.Time]float64)
	{
		baseEdb, e := models.GetChartEdbMappingByEdbInfoId(tableConfig.BaseEdbInfoId)
		if e != nil {
			err = fmt.Errorf("获取持仓相关性, 标的指标mapping信息失败, %v", e)
			return
		}
		baseEdbData := make([]*models.EdbDataList, 0)
		baseEdbData, e = models.GetEdbDataList(baseEdb.Source, baseEdb.SubSource, baseEdb.EdbInfoId, "", "")
		if e != nil {
			err = fmt.Errorf("获取标的指标数据失败, %v", e)
			return
		}
		for _, v := range baseEdbData {
			t, e := time.ParseInLocation(utils.FormatDate, v.DataTime, time.Local)
			if e != nil {
				continue
			}
			baseDateData[t] = v.Value
		}
	}

	// 获取品种最新数据日期
	baseDate, e := GetTradeClassifyNewestDataTime(tableConfig.Exchange, []string{tableConfig.ClassifyName})
	if e != nil {
		err = fmt.Errorf("获取持仓品种最新数据日期失败, %v", e)
		return
	}

	// 根据配置取出需要查询的合约
	contracts, e := GetTopContractsByType(tableConfig.ContractType, tableConfig.Exchange, tableConfig.ClassifyName, baseDate)
	if e != nil {
		err = fmt.Errorf("获取持仓相关性合约失败, %v", e)
		return
	}

	// [合约+期货公司]持仓数据
	tradeData, e := GetCorrelationTableTradeData(tableConfig.Exchange, tableConfig.ClassifyName, contracts, tableConfig.CompanyNames, tableConfig.PredictRatio, tableConfig.ContractType)
	if e != nil {
		err = fmt.Errorf("获取[合约+期货公司]持仓数据失败, %v", e)
		return
	}

	// 持仓数据转为指标
	tradeEdbList, tradeEdbDataMap, e := TransTradeData2EdbData(tradeData, tableConfig.ContractPosition)
	if e != nil {
		err = fmt.Errorf("[合约+期货公司]持仓数据转为指标失败, %v", e)
		return
	}

	// 遍历相关性配置(目前最多有两个滚动相关性), 遍历指标, 分别计算滚动相关性
	maxPre := 10 // 需要展示的以标的指标日期为准往前推N个交易日的滚动相关性
	startBase := baseDate.AddDate(0, 0, -maxPre)

	// 从20个自然日里截取11个数据对应的交易日，且当日排最前，后面依次为0、-1到-10
	rowDays := utils.GetTradingDays(baseDate.AddDate(0, 0, -20), baseDate)
	rowDays = utils.ReverseTimeSlice(rowDays)
	rowDays = rowDays[:11]
	for _, rc := range tableConfig.RollConfig {
		tableData := new(tradeAnalysisModel.CorrelationTableData)
		tableData.CalculateValue = rc.CalculateValue
		tableData.LeadValue = rc.LeadValue
		tableData.RowsData = make([]*tradeAnalysisModel.CorrelationTableRowData, 0)

		// 计算滚动相关性的开始日期, 用计算窗口往前推（可以往前多推俩月否则可能取不到）
		startDate := startBase.AddDate(0, 0, -(rc.CalculateValue + rc.LeadValue + 60))

		for kd, kv := range tradeEdbList {
			row := new(tradeAnalysisModel.CorrelationTableRowData)
			row.Exchange = kv.Exchange
			row.CompanyName = kv.CompanyName
			row.ClassifyName = kv.ClassifyName
			row.ClassifyType = kv.ClassifyType
			// 数据为合约加总时名称为品种名+方向，否则为合约+期货公司+方向
			if kv.IsTotal {
				row.RowName = fmt.Sprintf("%s%s%s", kv.ClassifyName, kv.CompanyName, tradeAnalysisModel.WarehouseTypeSuffixNames[kv.ContractPosition])
			} else {
				row.RowName = fmt.Sprintf("%s%s%s", kv.ClassifyType, kv.CompanyName, tradeAnalysisModel.WarehouseTypeSuffixNames[kv.ContractPosition])
			}
			row.DayData = make([]*tradeAnalysisModel.CorrelationTableRowDayData, 0)

			// 这里加行数据
			rd, e := CalculateRollCorrelationData(rc.CalculateValue, rc.LeadValue, baseDateData, tradeEdbDataMap[kd], startDate, baseDate)
			if e != nil {
				err = fmt.Errorf("计算行数据-滚动相关性失败, %v", e)
				return
			}
			for day, dt := range rowDays {
				// 这里展示要加负号
				var showDay int
				if day == 0 {
					showDay = 0
				} else {
					showDay = -day
				}
				row.DayData = append(row.DayData, &tradeAnalysisModel.CorrelationTableRowDayData{
					Day:      showDay,
					DataDate: dt.Format(utils.FormatDate),
					DataVal:  rd[dt], // 对应日期无值的话就是0，无相关性
				})
			}
			tableData.RowsData = append(tableData.RowsData, row)
		}
		tables = append(tables, tableData)
	}
	return
}

// GetTopContractsByType 根据配置获取需要取的合约
func GetTopContractsByType(contractType int, exchange, classifyName string, baseDate time.Time) (contracts []string, err error) {
	// 查询基准日期TOP20合约排名, 根据类型取出合约数
	exchangeContracts := make(map[string][]*tradeAnalysisModel.ContractTopRankData) // 交易所最终取的合约
	var contractMax int                                                             // 需要取出的品种对应的最大合约数
	classifyMax := make(map[string]int)                                             // 品种对应的合约数
	switch contractType {
	case tradeAnalysisModel.ContractQueryTypeTop:
		contractMax = 1
	case tradeAnalysisModel.ContractQueryTypeTop2:
		contractMax = 2
	case tradeAnalysisModel.ContractQueryTypeTop3:
		contractMax = 3
	case tradeAnalysisModel.ContractQueryTypeAll, tradeAnalysisModel.ContractQueryTypeTotal:
		contractMax = 999
	}

	// 遍历交易所, 查询各品种下的合约排名情况及TOP当日的多空单数据
	contractRanks, e := GetTopContractRank(exchange, []string{classifyName}, baseDate)
	if e != nil {
		err = fmt.Errorf("获取基准日期合约排名失败, %v", e)
		return
	}

	// ps.正常来讲这里查出来的合约是唯一的, 根据品种分组, 取出ContractType所需的合约数
	for _, v := range contractRanks {
		if classifyMax[v.ClassifyName] >= contractMax {
			continue
		}
		classifyMax[v.ClassifyName] += 1
		if exchangeContracts[v.Exchange] == nil {
			exchangeContracts[v.Exchange] = make([]*tradeAnalysisModel.ContractTopRankData, 0)
		}
		contracts = append(contracts, v.ClassifyType)
	}
	return
}

// CalculateRollCorrelationData 计算滚动相关性
func CalculateRollCorrelationData(calculateValue, leadValue int, baseEdbData map[time.Time]float64, changeEdbData map[time.Time]int, startDate, endDate time.Time) (dateRatio map[time.Time]float64, err error) {
	dateRatio = make(map[time.Time]float64)

	// 计算窗口，不包含第一天
	startDate = startDate.AddDate(0, 0, 1)

	// 平移变频指标
	changeDataMap := MoveDataDaysToNewDataList(changeEdbData, leadValue)

	// 计算计算时，需要多少个日期内数据
	calculateDay := calculateValue

	var minRatio, maxRatio float64
	// 计算 每个日期的相关性值
	startDateTime := startDate
	endDateTime := endDate.AddDate(0, 0, -(calculateDay - 1))

	// 是否开始第一条数据
	var isStart, isNotFirst bool
	for currDay := startDateTime; !currDay.After(endDateTime); currDay = currDay.AddDate(0, 0, 1) {
		yCalculateData := make([]float64, 0)
		baseCalculateData := make([]float64, 0)

		// 取出对应的基准日期的值
		for i := 0; i < calculateDay; i++ {
			iDay := currDay.AddDate(0, 0, i)

			tmpBaseValue, ok1 := baseEdbData[iDay]
			tmpChangeValue, ok2 := changeDataMap[iDay]
			if !ok1 || !ok2 {
				continue
			}
			baseCalculateData = append(baseCalculateData, tmpBaseValue)
			yCalculateData = append(yCalculateData, tmpChangeValue)
		}

		// 没有数据的话，那就不返回
		if len(baseCalculateData) == 0 {
			continue
		}
		// 公式计算出领先/滞后频度对应点的相关性系数
		ratio := utils.CalculateCorrelationByIntArr(baseCalculateData, yCalculateData)

		// 过滤前面都是0的数据
		{
			if ratio != 0 {
				isStart = true
			}

			if !isStart {
				continue
			}
		}

		dataTime := currDay.AddDate(0, 0, calculateDay-1)

		// 保留4位小数
		ratio = math.Round(ratio*10000) / 10000
		dateRatio[dataTime] = ratio

		if !isNotFirst {
			minRatio = ratio
			maxRatio = ratio
			isNotFirst = true
		}
		if minRatio > ratio {
			minRatio = ratio
		}
		if maxRatio < ratio {
			maxRatio = ratio
		}
	}
	return
}

// MoveDataDaysToNewDataList 平移指标数据生成新的数据序列
func MoveDataDaysToNewDataList(originDateData map[time.Time]int, moveDay int) (newDateData map[time.Time]float64) {
	// 取出最早最晚的日期
	var minDate, maxDate time.Time
	for k, _ := range originDateData {
		if minDate.IsZero() || k.Before(minDate) {
			minDate = k
		}
		if maxDate.IsZero() || k.After(maxDate) {
			maxDate = k
		}
	}

	// 处理领先、滞后数据
	newDateMap := make(map[time.Time]float64)
	for currDate, value := range originDateData {
		newDate := currDate.AddDate(0, 0, moveDay)
		newDateMap[newDate] = float64(value)
	}
	minDate = minDate.AddDate(0, 0, moveDay)
	maxDate = maxDate.AddDate(0, 0, moveDay)

	// 创建一个有序的数据序列, 用于处理找不到数据时取前个交易日值的情况
	type OrderedData struct {
		DataTime time.Time
		Val      float64
	}
	newDataList := make([]*OrderedData, 0)

	// 获取日期相差日
	dayNum := utils.GetTimeSubDay(minDate, maxDate)
	newDateData = make(map[time.Time]float64)
	for i := 0; i <= dayNum; i++ {
		currDate := minDate.AddDate(0, 0, i)
		tmpValue, ok := newDateMap[currDate]
		if !ok {
			//找不到数据，那么就用前面的数据吧
			if len(newDataList)-1 < 0 {
				tmpValue = 0
			} else {
				tmpValue = newDataList[len(newDataList)-1].Val
			}
		}
		tmpData := &OrderedData{
			DataTime: currDate,
			Val:      tmpValue,
		}
		newDateData[currDate] = tmpData.Val
		newDataList = append(newDataList, tmpData)
	}
	return
}
